# Maverick - Self Driving Car

This project was developed as part of the [Self Driving Challenge](https://www.selfdrivingchallenge.nl/) 2024, within NHL Stenden University of Applied Sciences in Emmen, The Netherlands.

The repo contains the source code which operates and enables a kart to drive autonomously while completing a set amount of challenges.

```
                 __  __                      _      _
                |  \/  |                    (_)    | |
                | \  / | __ ___   _____ _ __ _  ___| | __
                | |\/| |/ _' \ \ / / _ \ '__| |/ __| |/ /
                | |  | | (_| |\ V /  __/ |  | | (__|   <
                |_|  |_|\__,_| \_/ \___|_|  |_|\___|_|\_\

                                   ___,,___
                                ,d8888888888b,_
                            _,d889''        8888b,
                        _,d8888''          8888888b,
                    _,d8889''           888888888888b,_
                _,d8889''             888888889''688888, /b
            _,d8889''               88888889''     `6888d 6,_
         ,d88886''              _d888889''           ,8d  b888b,  d\
       ,d889''888,             d8889''               8d   9888888Y  )
     ,d889''   `88,          ,d88''                 d8    `,88aa88 9
    d889''      `88,        ,88''                   `8b     )88a88''
   d88''         `88       ,88                   88 `8b,_ d888888
  d89            88,      88                  d888b  `88`_  8888
  88             88b      88                 d888888 8: (6`) 88'')
  88             8888b,   88                d888aaa8888, `   ''Y''
  88b          ,888888888888                 `d88aa `88888b ,d8
  `88b       ,88886 `88888888                 d88a  d8a88` `8/
   `q8b    ,88''`888  `888''"`88          d8b  d8888,` 88/ 9)_6
     88  ,88"   `88  88p    `88        d88888888888bd8( Z~/
     88b 8p      88 68''      `88      88888888'' `688889`
     `88 8        `8 8,       `88    888 `8888,   `qp''
       8 8,        `q 8b       `88  88"    `888b
       q8 8b        "888        `8888''
        "888                     `q88b
                                  "888''
```

## Setup Specifications

- Operating System: [Ubuntu 22.04 LTS](https://releases.ubuntu.com/jammy/)
- Programming Language: Python 3.10 (or higher)
- Dependencies: _installed automatically by running the **'maverick_setup.bash'** file_
- Software: [ROS2 Humble](https://docs.ros.org/en/humble/index.html)

## Installation

To install and run this project, follow these steps:

1. Clone the repository:

```bash
git clone https://gitlab.com/nhlstendenIT/rdw/rdw-maveric
```

2. Navigate to the project directory: `cd rdw-maveric`
3. Install the dependencies:

```bash
sudo ./maverick_setup.bash
```

- **If not sure what the prompts mean, just keep pressing _[Enter]_ / _(yes)_ until the installation finishes.**

4. **Optional:** For development, you might have to configure a virtual CAN bus. Run `sudo ./vcan_setup.bash` for a simple installation _(you need to run the setup script every time you restart the machine)_.
5. Build the project:

```bash
# First switch to the actual workspace (within folder 'rdw-maveric')
cd maverick_ws

# Inside the maverick workspace build the project
colcon build --symlink-install

# Once the build is finished, source the environment
source install/local_setup.bash
```

6. That's it! Now you should be able to run the available ROS2 nodes and launch files!

## Usage

**Make sure you have successfully completed the steps mentioned in [Installation](#installation) first!**

To run a node or launch file, use the ROS2 syntax. For example, to run the simulation launch script, use the following command:

```bash
ros2 launch gazebo_sim rsp_sim.launch.py
```

This script will load a simulation environment in an application called Gazebo. The simulation environment will contain a geometrical representation of the real kart.

To control the kart's movement, run the following node **in another terminal** while keeping the launch script running!

```bash
ros2 run teleop_twist_keyboard teleop_twist_keyboard
```

_This node will allow you to control the simulated kart using the keys on your keyboard._

### _Note!_

If after the first couple of times you try to run the launch file the above simulation environment does not start properly, or you get errors, do the following steps:

- Stop the launch script
- Start **only** Gazebo, manually. You can run the following command in the terminal: `gazebo`.
- Wait for Gazebo to load. After it finished loading, stop Gazebo _(Ctrl + C)_.
- Re-run the launch script.

The kart can also be controlled by publishing _Twist_ messages to the `/cmd_vel` topic.

To run the whole simulation environment, together with the core package (which contains the node responsible for completing the challanges) run the `maverick_sim.launch.py` file inside the **core** package:

```bash
ros2 launch core maverick_sim.launch.py
```

The launch file meant for the kart itself can be started as following:

```bash
ros2 launch core maverick.launch.py
```
