#!/bin/bash

# Get Ubuntu version
ubuntu_version=$(lsb_release -rs)

# Check if Ubuntu version is 22.04
if [ "$ubuntu_version" != "22.04" ]; then
    echo "Error: Ubuntu version is not 22.04. Exiting..."
    exit 1
fi

# Update and Upgrade
sudo apt update -y && sudo apt upgrade -y && sudo apt autoremove

echo "======================================================="
echo "Installing ROS"
echo "======================================================="

# Set locale
sudo apt update && sudo apt install locales
sudo locale-gen en_US en_US.UTF-8
sudo update-locale LC_ALL=en_US.UTF-8 LANG=en_US.UTF-8
export LANG=en_US.UTF-8

# Setup sources
# First ensure that the Ubuntu Universe repository is enabled
sudo apt install software-properties-common
sudo add-apt-repository universe

# Add the ROS 2 GPG key with apt
sudo apt update && sudo apt install curl -y
sudo curl -sSL https://raw.githubusercontent.com/ros/rosdistro/master/ros.key -o /usr/share/keyrings/ros-archive-keyring.gpg

# Add the repository to your sources list
echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/ros-archive-keyring.gpg] http://packages.ros.org/ros2/ubuntu $(. /etc/os-release && echo $UBUNTU_CODENAME) main" | sudo tee /etc/apt/sources.list.d/ros2.list > /dev/null

# Update and upgrade again before installing ros2 packages
sudo apt update -y && sudo apt upgrade -y && sudo apt autoremove -y

# Install ROS
sudo apt install ros-humble-desktop

# Install Ros dev tools
sudo apt install ros-dev-tools

echo "======================================================="
echo "Ros was installed!"
echo "======================================================="

# Prompt user for approval
read -r -p "Do you want to add ROS ro the shell startup script (recommended)? (y/n): " response

# Convert response to lowercase
response_lower=$(echo "$response" | tr '[:upper:]' '[:lower:]')

# Check if user approved
if [ "$response_lower" = "yes" ] || [ "$response_lower" = "y" ] || [ -z "$response" ]; then
    echo "source /opt/ros/humble/setup.bash" >> ~/.bashrc
fi

# export ROS_DOMAIN_ID to 73
echo "export ROS_DOMAIN_ID=73" >> ~/.bashrc

# Installing colcon
echo "======================================================="
echo "Installing colcon"
echo "======================================================="

sudo apt install python3-colcon-common-extensions

# Enable autocompletion for colcon
echo "source /usr/share/colcon_argcomplete/hook/colcon-argcomplete.bash" >> ~/.bashrc

echo "======================================================="
echo "Colcon installed"
echo "======================================================="

echo "======================================================="
echo "Installing gazebo (and other usefull simulation tools)"
echo "======================================================="

sudo apt install ros-humble-urdf-tutorial
sudo apt install ros-humble-gazebo-ros-pkgs
sudo apt install ros-humble-rplidar-ros

# Install pip
sudo apt install python3-pip
# Install setuptools 58.2.0 to avoid gazebo installation error
pip3 install setuptools==58.2.0

# Install other python dependencies
pip3 install -r requirements.txt

echo "======================================================="
echo "Gazebo installed"
echo "======================================================="

echo "======================================================="
echo "Installing ros2_control"
echo "======================================================="

sudo apt install ros-humble-ros2-control ros-humble-ros2-controllers ros-humble-gazebo-ros2-control

echo "======================================================="
echo "Ros2_control installed"
echo "======================================================="

# Source the bashrc file
source ~/.bashrc

echo ""
echo "======================================================="
echo "======================================================="
echo "======================================================="
echo "
  __  __                      _      _    
 |  \/  |                    (_)    | |   
 | \  / | __ ___   _____ _ __ _  ___| | __
 | |\/| |/ _' \ \ / / _ \ '__| |/ __| |/ /
 | |  | | (_| |\ V /  __/ |  | | (__|   < 
 |_|  |_|\__,_| \_/ \___|_|  |_|\___|_|\_\
"
 
echo ""
echo '
                                    ___,,___
                                ,d8888888888b,_
                            _,d889''        8888b,
                        _,d8888''          8888888b,
                    _,d8889''           888888888888b,_
                _,d8889''             888888889''688888, /b
            _,d8889''               88888889''     `6888d 6,_
         ,d88886''              _d888889''           ,8d  b888b,  d\
       ,d889''888,             d8889''               8d   9888888Y  )
     ,d889''   `88,          ,d88''                 d8    `,88aa88 9
    d889''      `88,        ,88''                   `8b     )88a88''
   d88''         `88       ,88                   88 `8b,_ d888888
  d89            88,      88                  d888b  `88`_  8888
  88             88b      88                 d888888 8: (6`) 88'')
  88             8888b,   88                d888aaa8888, `   ''Y''
  88b          ,888888888888                 `d88aa `88888b ,d8
  `88b       ,88886 `88888888                 d88a  d8a88` `8/
   `q8b    ,88''`888  `888''"`88          d8b  d8888,` 88/ 9)_6
     88  ,88"   `88  88p    `88        d88888888888bd8( Z~/
     88b 8p      88 68''      `88      88888888'' `688889`
     `88 8        `8 8,       `88    888 `8888,   `qp''
       8 8,        `q 8b       `88  88"    `888b
       q8 8b        "888        `8888''
        "888                     `q88b
                                  "888''
'
echo "======================================================="
echo "======================================================="
echo "======================================================="

echo ""
echo "Installation complete"
echo "Before running ros2, remember to source the environment (unless you added it to the startup script). "
echo "You can use the following command: 'source /opt/ros/humble/setup.bash'"
echo ""
echo "Before running a node or a launch file, build the code using 'colcon'"
echo "Use the following command: 'colcon build --symlink-install'"
echo ""
echo "And then source the environment using 'source install/local_setup.bash'"
echo "THE ABOVE COMMAND MUST BE WITHIN THE 'rdw-maveric/maverick_ws' FOLDER"
echo ""
echo "If you plan to debug the code, you might have to run the 'vcan_setup.bash' script to setup the virtual can interface"
echo ""
echo ""
echo "Enjoy your ROS2 setup!"


