from .interface import CanBusInterface
from .brake_feedback import BrakeFeedback

import time


class Brake(CanBusInterface):
    def __init__(self, logger, brake_feedback: BrakeFeedback) -> None:
        super().__init__(logger=logger, arbitration_id=0x110)
        self.logger.info("Brake Interface Initialized")

        self._brake_feedback = brake_feedback

    def set_brake(self, brake: bool) -> None:
        # self.logger.info(f"Received braking value: {brake}")

        # The brake value (byte 1 or index 0) can be any value between 0 and 100,
        # and represents the percentage of the maximum brake force.
        data = [0, 0, 0, 0, 0, 0, 0, 0]

        # Wait for the brake feedback to be ready
        while not self._brake_feedback.is_idle():
            time.sleep(0.05)

        # Perform absolute braking
        if brake and self._brake_feedback.is_not_braking():
            data[0] = 100

        # Set the brake if the new brake value is different from the current brake value
        if data[0] != self._message.data[0]:
            self.set_bus_data(data)
            self.send_bus_message()
            self.logger.info(f"Setting brake to {data[0]}%")
