from rclpy.impl.rcutils_logger import RcutilsLogger
from .interface import CanBusInterface

import can


class BrakeFeedback(CanBusInterface, can.Listener):
    def __init__(self, logger: RcutilsLogger) -> None:
        super().__init__(logger, arbitration_id=0x710)
        self.logger.info("Brake Feedback Interface Initialized")

        # Define the attributes
        self._current_potentiometer_value: int = 0
        self._target_potentiometer_value: int = 0
        self._direction: int = 0
        self._speed: int = 0
        self._error: int = 0

    def on_message_received(self, msg: can.Message) -> None:
        if msg.arbitration_id == self.arbitration_id:
            # Data format: [
            #               current_potentiometer_value, current_potentiometer_value,
            #               target_potentiometer_value, target_potentiometer_value
            #               direction (1 - retracting, 0 - extending),
            #               speed, error (0 - no error, 1 - error), 0
            #           ]

            # Get the byte array from the message
            data = msg.data

            first_byte = data[0]
            second_byte = data[1]
            thrid_byte = data[2]
            fourth_byte = data[3]

            # Current potentiometer value is the compound value of bytes 1 and 2
            self._current_potentiometer_value = (first_byte << 8) | second_byte

            # Target potentiometer value is the compound value of bytes 3 and 4
            self._target_potentiometer_value = (thrid_byte << 8) | fourth_byte

            self._direction = data[4]
            self._speed = data[5]
            self._error = data[6]

    def is_idle(self) -> bool:
        return self._current_potentiometer_value == self._target_potentiometer_value

    def is_not_braking(self) -> bool:
        return self._current_potentiometer_value == 0
