from control import CanBus
from rclpy.impl.rcutils_logger import RcutilsLogger

import can
import struct


class CanBusInterface(object):
    """
    This class gives access to the CAN bus interface for the vehicle.
    """

    CAN_MESSAGE_SENDING_SPEED = 0.04

    def __init__(self, logger: RcutilsLogger, arbitration_id: int) -> None:
        # Get the logger for ROS2 logging
        self._logger = logger
        self._arbitration_id = arbitration_id

        # Initialize the CAN bus interface
        can_bus = CanBus()
        self._can_bus: can.Bus = can_bus.bus

        # Create a CAN bus message
        self._message = can.Message(
            arbitration_id=arbitration_id,
            is_extended_id=False,
            data=[0, 0, 0, 0, 0, 0, 0, 0]
        )

        # Create a task for sending periodic messages
        self._bus_task = self._can_bus.send_periodic(self._message, self.CAN_MESSAGE_SENDING_SPEED)

    @property
    def can_bus(self) -> CanBus:
        return self._can_bus

    @property
    def arbitration_id(self) -> int:
        return self._arbitration_id

    @property
    def logger(self) -> RcutilsLogger:
        return self._logger

    def set_bus_data(self, data: list) -> None:
        self._message.data = data

    def send_bus_message(self) -> None:
        # self._can_bus.send(self._message)
        # self._can_bus.send_periodic(self._message, period=0.1, duration=0.15)
        self._bus_task.modify_data(self._message)

    @staticmethod
    def convert_to_bytes(value: float, byteorder: str = 'big') -> bytearray:
        if byteorder.lower() not in ['big', 'little']:
            raise ValueError("byteorder must be either 'big' or 'little'")

        byteorder = '<f' if byteorder == 'little' else '>f'

        # Make sure the value is a float
        value = float(value)

        # return (value).to_bytes(2, byteorder=byteorder.lower())
        return bytearray(struct.pack(byteorder, value))
