from rclpy.node import Publisher
from std_msgs.msg import Float32

from .interface import CanBusInterface

import can


class SpeedSensor(CanBusInterface, can.Listener):
    def __init__(self, logger, speed_publisher: Publisher) -> None:
        super().__init__(logger, arbitration_id=0x440)
        self.logger.info("Speed Sensor Interface Initialized")

        self._speed_publisher = speed_publisher
        self._msg = Float32()
        self.speed = 0.0

    @property
    def speed(self) -> float:
        return self._speed

    @speed.setter
    def speed(self, speed: float) -> None:
        self._speed = speed

    def on_message_received(self, message: can.Message) -> None:
        # Filter the messages based on the arbitration ID
        if message.arbitration_id == self.arbitration_id:
            # The compound value of bytes 1 and 2 represents the current speed of the vehicle
            first_byte = message.data[0]
            second_byte = message.data[1]
            # received_speed = (message.data[1] << 8) | message.data[0]
            received_speed = (first_byte << 8) | second_byte

            # self.logger.info(f"Converted speed (HM/h) is: {received_speed}")

            # Convert the speed from HM/h to m/s
            actual_speed = received_speed / 36

            # Publish the speed message
            self._msg.data = actual_speed
            self._speed_publisher.publish(self._msg)

            # Log only if the speed is different from the previous one
            if self.speed != actual_speed:
                self.speed = actual_speed

                self.logger.info(
                    f"Received new speed message: {actual_speed}\n"
                )
