from .interface import CanBusInterface

import can


class SteeringSensor(CanBusInterface, can.Listener):
    """
    Returns 'ticks' instead of EXCPECTED angles
    In other words: USELESS
    """

    def __init__(self, logger) -> None:
        super().__init__(logger, arbitration_id=0x1E5)

        self.logger.info("Steering Sensor Interface Initialized")

        self._angle = 0.0

    @property
    def angle(self) -> float:
        return self._angle

    @angle.setter
    def angle(self, angle: float) -> None:
        self._angle = angle

    def on_message_received(self, message: can.Message) -> None:
        # Filter the messages based on the arbitration ID
        if message.arbitration_id == self.arbitration_id:
            # The compound value of bytes 1 and 2 represents
            # the current angle of the steering wheel
            angle = (message.data[1] << 8) | message.data[0]

            self.angle = angle

            # Log only if the angle is different from the previous one
            if self.angle != angle:
                self.logger.info(f"Received angle message: {angle}")
