import time
import unittest

import launch
import launch.actions
import launch_testing.actions
import launch_testing.markers
import pytest
import rclpy

# from rclpy.node import Node

try:
    from control.controller_manager import ControlManager
except Exception:
    from ..control.controller_manager import ControlManager


@pytest.mark.launch_test
@launch_testing.markers.keep_alive
def generate_test_description():
    return launch.LaunchDescription([
        launch.actions.ExecuteProcess(
            cmd=['ros2', 'run', 'control', 'control_manager'],
            output='screen'
        ),

        launch_testing.actions.ReadyToTest()
    ])


class TestController(unittest.TestCase):
    def start_spin(self):
        rclpy.spin_once(self.node, timeout_sec=0.1)

    def setUp(self) -> None:
        rclpy.init()
        # self.node = Node("testing_node")
        self.node = ControlManager()

        rclpy.spin(self.node)

        return super().setUp()

    def tearDown(self) -> None:
        self.node.destroy_node()
        rclpy.shutdown()
        return super().tearDown()

    def test_start_control_manager(self):
        start = time.time()
        found = False

        while time.time() - start < 10.0:
            if self.node.controller is not None:
                found = True
                break
            time.sleep(1.0)

        assert found, "Control manager not started"

    def test_ackerman_controller(self):
        linear_x = 5.0
        angular_z = 1.0

        # Test steering angle
        front_steering_angle = self.node.controller._calculate_front_steering_angle(
            linear_x, angular_z
        )

        back_wheel_speed = self.node.controller._calculate_back_wheel_speed(
            linear_x, front_steering_angle
        )

        assert round(front_steering_angle, 2) == 0.21
        assert round(back_wheel_speed, 2) == 5.12

    def test_brake(self):
        linear_x = 0.0
        angular_z = 1.0

        # Test steering angle
        self.node.controller.process_velocity(linear_x, angular_z)

        # Test brake
        assert self.node.controller._brake._message.data[0] == 100

        # Test no brake
        linear_x = 5.0
        angular_z = 1.0

        # Test steering angle
        self.node.controller.process_velocity(linear_x, angular_z)

        # Test brake
        assert self.node.controller._brake._message.data[0] == 0
