from typing import Optional

from rclpy.node import Node
from cv_bridge import CvBridge
from sensor_msgs.msg import Image

from .calibrate_tools import CalibrationTools
from core.utils import display_image_safe

import rclpy


class CalibrationNode(Node):
    def __init__(self):
        super().__init__('calibration_lane_tracking_node')
        self.get_logger().info('CalibrationNode started')

        # Subscribe to the /camera_front/image_raw topic
        self.create_subscription(
            Image,
            '/camera_front/image_raw',
            self._image_front_callback,
            10
        )
        self._front_image: Optional[Image] = None

        # Subscribe to the /camera_left/image_raw topic
        self.create_subscription(
            Image,
            '/camera_left/image_raw',
            self._image_left_callback,
            10
        )
        self._left_image: Optional[Image] = None

        # Subscribe to the /camera_right/image_raw topic
        self.create_subscription(
            Image,
            '/camera_right/image_raw',
            self._image_right_callback,
            10
        )
        self._right_image: Optional[Image] = None

        self._cv_bridge = CvBridge()
        self._calibration_tools = CalibrationTools()

    def _image_front_callback(self, msg: Image) -> None:
        # Process the Image message
        self._front_image = self._cv_bridge.imgmsg_to_cv2(msg)

        # display_image_safe(self._front_image, "Front Camera")

        self._calibration_tools.front_cam = self._front_image
        self._calibration_tools.calibrate_all_cameras()

    def _image_left_callback(self, msg: Image) -> None:
        # Process the Image message
        self._left_image = self._cv_bridge.imgmsg_to_cv2(msg)
        self._calibration_tools.left_cam = self._left_image

    def _image_right_callback(self, msg: Image) -> None:
        # Process the Image message
        self._right_image = self._cv_bridge.imgmsg_to_cv2(msg)
        self._calibration_tools.right_cam = self._right_image


def main(args=None):
    rclpy.init(args=args)
    node = CalibrationNode()
    rclpy.spin(node)
    node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
