from rclpy.clock import Clock
from simple_pid import PID

# IMPORTANT!!!
# -------------------------
# TODO Adjust
# *** NECESSARY_DIST_TO_WALL : change the float (in meters) as barriers on the track are flat and not sloped as in simulation. Distance irl is shorter to them.
# *** ALL OCCURENCES OF | self.dist_traveled : outcomment if the timer is too short and the car travels to short of a distance. If the distance is longer, do not worry as it has lidar distance checks for it
# *** self.Kp | self.Ki | self.Kd : change if it wiggles too much on the angular.z
# *** degree_average_rx | line 192 | ifstatement() : change if the car does not park in the right spot. it says 2.7 meters but might differ depends on the barrier position and thiccness
# *** NUM_OF_LX_KEYS_TO_STOP = 24 | line 139 | change between 18, 21, 24 to adjust the sequence 1 stopping point and the curve of the car.
# *** prev_angle_1 = self.seq_2_mem_buffer[8] | prev_angle_2 = self.seq_2_mem_buffer[9] | Line 166, 167 | change the indices, the smaller the integer in the [], the shorter the length of the sequence 2 and the final curve of the parking gets smaller.
# -------------------------


class ParallelMethods(object):
    MEMORY_BUFFER_LEN = 10  # amount of past readings from lidar to be saved
    SCAN_LASER_WIDTH = 10  # If this Num of keys in sorted dict -> there is obj on the right
    # how many times in a row objects have to be detected on the right
    NUM_OF_SCANS_TO_DETECT = 5
    CAR_LEN = 1.9  # Car length
    QUARTER_A_CAR_LENGTH = CAR_LEN * 0.25  # For Mathew <3
    NECESSARY_DIST_TO_WALL = 1.5  # Desired distance to the wall
    # Used in sequence 1 to stop the car at the right angle - 45 degrees
    NUM_OF_LX_KEYS_TO_STOP = 21

    def __init__(self, clock: Clock):
        # Find parking spot flags & variables
        self.is_barrier_1_detected = False
        self.is_gap_detected = False
        self.is_barrier_2_detected = False
        self.seq_2_memory_values = [{}] * self.MEMORY_BUFFER_LEN
        self.is_parking_in_progress = False

        # Clock variables
        self._clock = clock  # Clock obj -> get time from
        self._start_time = None

        # Starting velocities
        self.current_linear_velocity = 1.0
        self.current_angular_velocity = 0.0

        # Parking sequence support
        self.is_car_ready_to_park = False
        self.dist_traveled = 0.0

        # Parking sequence flags
        self._parking_sequence_1_finished = False
        self._parking_sequence_2_finished = False
        self._parking_sequence_3_finished = False
        self.seq_2_mem_buffer = [None] * self.MEMORY_BUFFER_LEN

        # Distance To the wall
        # PID controller for steering
        self.Kp = 6.0
        self.Ki = 3.0
        self.Kd = 2.0
        self.pid = PID(self.Kp, self.Ki, self.Kd,
                       setpoint=self.NECESSARY_DIST_TO_WALL)
        self.pid.output_limits = (-1, 1)
        self.pid.sample_time = 0.1
        self.previous_distance = None
        self.distance_threshold = 0.15

    def _add_to_memory_buffer(self, scan_dict: dict):
        if len(self.seq_2_memory_values) >= self.MEMORY_BUFFER_LEN:
            self.seq_2_memory_values.pop()

        self.seq_2_memory_values.insert(0, scan_dict)

    def _add_to_seq_2_memory_buffer(self, dist: float):
        if len(self.seq_2_mem_buffer) >= self.MEMORY_BUFFER_LEN:
            self.seq_2_mem_buffer.pop()

        self.seq_2_mem_buffer.insert(0, dist)

    # Wrapper method. Call on each lidar scan
    def detect_parking_spot(self, scan_dict: dict):
        # Get values only from the right and save to buffer
        self._filter_right_values(scan_dict)
        filtered_dict_rx = self._filter_dict_by_key_range(scan_dict, 810, 846)
        filtered_dict_rx_empty = self._filter_dict_by_key_range(
            scan_dict, 900, 936
        )

        # Firstly try to detect barrier 1
        if self.is_barrier_1_detected is False and self.is_gap_detected is False and self.is_barrier_2_detected is False:
            # Try to detect 1st barrier by checking mem buffer
            self._detect_1_block()
            self.is_parking_in_progress = False

        # Secondly try to detect empty gap (parking space)
        if self.is_barrier_1_detected is True and self.is_gap_detected is False and self.is_barrier_2_detected is False:
            self.current_linear_velocity = 0.5
            print("Block 1 Detected")
            self.is_parking_in_progress = False
            self._detect_gap()

        # Thirdly try to detect barrier 2
        if self.is_barrier_1_detected is True and self.is_gap_detected is True and self.is_barrier_2_detected is False:
            # Wait for barrier 2 to appear
            print("Gap detected")
            self.is_parking_in_progress = True
            self._detect_2_block()

        # Set the timer to start driving
        if self.is_barrier_1_detected is True and self.is_gap_detected is True and self.is_barrier_2_detected is True and self._start_time is None:
            self.current_linear_velocity = 0.0
            print("Block 2 detected")
            self._start_time = self._clock.now().to_msg().sec

        # Drive 3/4 of car body past parking spot
        if self.is_barrier_1_detected is True and self.is_gap_detected is True and self.is_barrier_2_detected is True and self._start_time is not None:
            current_time = self._clock.now().to_msg().sec
            if self.is_car_ready_to_park is False:
                self.current_linear_velocity = 0.5
                time_passed = current_time - self._start_time
                if len(filtered_dict_rx) > 0 or len(filtered_dict_rx_empty) == 0:
                    if self.dist_traveled >= (self.CAR_LEN + self.QUARTER_A_CAR_LENGTH):
                        print('Car ready to park')
                        # Stop car when its 3/4 of its body past parking spot.
                        self.current_linear_velocity = 0.0
                        self.is_car_ready_to_park = True

                # Calculate dist travelled based on speed and time
                self.dist_traveled = self.current_linear_velocity * time_passed
                print(self.dist_traveled)

            self._add_to_memory_buffer(filtered_dict_rx_empty)

        self.keep_close_distance_to_wall(scan_dict)

    def _parking_sequence_1(self, scan_dict: dict):
        self._start_time = None
        # Start Driving Back and Right
        self.current_angular_velocity = 1.0  # turn wheels right
        self.current_linear_velocity = -0.5  # drive backwards

        # Get values of around LX & RX (Left/Right of seat)
        filtered_dict_lx = self._filter_dict_by_key_range(
            scan_dict, 53, (53 + self.NUM_OF_LX_KEYS_TO_STOP)
        )
        filtered_dict_rx = self._filter_dict_by_key_range(scan_dict, 923, 1026)

        # Check if both LX & RX see Barrier 1 & 2 Respectively
        if len(filtered_dict_rx) != 0 and len(filtered_dict_lx) >= self.NUM_OF_LX_KEYS_TO_STOP:
            # Stop the car after sequence 1 is done (~45) degrees to the wall)
            self.current_angular_velocity = 0.0
            self.current_linear_velocity = 0.0
            print('Sequence 1 finished')
            self._parking_sequence_1_finished = True

    def _parking_sequence_2(self, scan_dict: dict):
        # Move car back straight
        self.current_angular_velocity = 0.0
        self.current_linear_velocity = -0.5
        lx_1_degree = self._filter_dict_by_key_range(scan_dict, 56, 59)
        # Calculate average value for azimuth 17
        degree_average = sum(lx_1_degree.values(
        )) / len(lx_1_degree.values()) if len(lx_1_degree.values()) > 0 else 0

        # Check if new value is smaller than previous two -> decreasing trend
        prev_angle_1 = self.seq_2_mem_buffer[8]
        prev_angle_2 = self.seq_2_mem_buffer[9]
        if degree_average is not None and prev_angle_1 is not None and prev_angle_2 is not None:
            if degree_average >= prev_angle_1 and degree_average >= prev_angle_2:
                pass
            else:
                # value started growing -> hit the angle of the blocks
                self._parking_sequence_2_finished = True
                print('Sequence 2 finished')

        # Save to mem buffer
        self._add_to_seq_2_memory_buffer(degree_average)

    def _parking_sequence_3(self, scan_dict: dict):
        if self._start_time is None:
            self._start_time = self._clock.now().to_msg().sec
        if self.is_car_ready_to_park is True and self._parking_sequence_1_finished is True and self._parking_sequence_2_finished is True:
            self.current_angular_velocity = -1.0
            self.current_linear_velocity = -0.5
            current_time = self._clock.now().to_msg().sec

        rx_1_degree = self._filter_dict_by_key_range(scan_dict, 1023, 1026)
        lx_1_degree = self._filter_dict_by_key_range(scan_dict, 53, 56)

        degree_average_rx = (
            sum(rx_1_degree.values()) / len(rx_1_degree.values())
            if len(rx_1_degree.values()) > 0 else 0
        )
        degree_average_lx = (
            sum(lx_1_degree.values()) / len(lx_1_degree.values())
            if len(lx_1_degree.values()) > 0 else 0
        )
        time_passed = current_time - self._start_time

        if degree_average_rx is not None and degree_average_lx is not None:
            if self.dist_traveled >= 3.5 or degree_average_rx != 0 and degree_average_rx <= 2.7:
                self.current_linear_velocity = 0.0
                self.current_angular_velocity = 0.0
                print('Sequence 3 finished')
                self._parking_sequence_3_finished = True
                print('Parked')

        self.dist_traveled = abs(self.current_linear_velocity) * time_passed
        print(self.dist_traveled)

    def parking_sequence_wrapper(self, scan_dict: dict):
        if (
            self.is_car_ready_to_park is True and
            self._parking_sequence_1_finished is False and
            self._parking_sequence_2_finished is False and
            self._parking_sequence_3_finished is False
        ):
            self._parking_sequence_1(scan_dict)

        if (
            self.is_car_ready_to_park is True and
            self._parking_sequence_1_finished is True and
            self._parking_sequence_2_finished is False and
            self._parking_sequence_3_finished is False
        ):
            self._parking_sequence_2(scan_dict)

        if (
            self.is_car_ready_to_park is True and
            self._parking_sequence_1_finished is True and
            self._parking_sequence_2_finished is True and
            self._parking_sequence_3_finished is False
        ):
            self._parking_sequence_3(scan_dict)

    def keep_close_distance_to_wall(self, scan_dict: dict):
        if self.is_parking_in_progress is True:
            return

        # Calculate average distance for a specified key range
        right_side_keys = self._filter_dict_by_key_range(scan_dict, 826, 838)
        one_90_degree_angle = self._filter_dict_by_key_range(
            scan_dict, 810, 813
        )

        right_side_shortest_distance = (
            min(right_side_keys.values()) if len(right_side_keys) > 0 else 0
        )

        print(
            f"Right side shortest distance: {right_side_shortest_distance:.2f}"
        )

        one_90_degree_dist = (
            sum(one_90_degree_angle.values()) /
            len(one_90_degree_angle.values())
            if len(one_90_degree_angle.values()) > 0 else 0
        )
        print(f"90 degrees shortest distance: {one_90_degree_dist:.2f}")

        if right_side_shortest_distance <= 0:
            print("Invalid distance data.")
            return
        elif right_side_shortest_distance > 2.5:
            print("Distance data is too large.")
            return

        # Check for a big change in distance for 90 degree right side key 810
        if self.previous_distance is not None:
            distance_change = abs(one_90_degree_dist - self.previous_distance)
            if distance_change > self.distance_threshold:
                print(
                    f"Big distance change detected, IGNORING: {distance_change:.2f}"
                )
                self.current_angular_velocity = 0.0
                return

        # If the desired distance gets reached NEC_DIST_TO_WALL, stop the steering
        if one_90_degree_dist == self.NECESSARY_DIST_TO_WALL:
            self.current_angular_velocity = 0.0
            print(
                f"Distance: {one_90_degree_dist:.2f} - Target distance reached. Steering stopped."
            )
            return

        self.previous_distance = one_90_degree_dist

        # Pid adjustment for steering angle. Decreased by 0.5 to reduce the steering angle
        pid_adjustment = self.pid(right_side_shortest_distance)
        pid_adjustment *= 0.5

        # Calculate the new steering angle
        new_steering_angle = self.current_angular_velocity + pid_adjustment
        print('new_steering_angle: ', new_steering_angle)

        # Apply safety bounds to steering (max angles for the car)
        max_steering_angle = 1.0
        min_steering_angle = -1.0
        new_steering_angle = max(
            min(new_steering_angle, max_steering_angle),
            min_steering_angle
        )

        # Set the new steering angle
        self.current_angular_velocity = new_steering_angle
        print(
            f"Distance: {right_side_shortest_distance:.2f} Steering: {new_steering_angle:.2f}"
        )

    def _detect_gap(self):
        detect_in_row = 0
        prev_detected = True
        for history in self.seq_2_memory_values:
            if self._check_for_empty_space(history):
                if prev_detected is True:
                    detect_in_row += 1
                prev_detected = True
            else:
                prev_detected is False

        if detect_in_row >= self.NUM_OF_SCANS_TO_DETECT:
            self.is_gap_detected = True

    def _detect_1_block(self):
        detected_in_row = 0
        prev_detected = True
        # Iterate over all elements of mem buffer
        for history in self.seq_2_memory_values:
            if self._check_if_obj_right(history):
                if prev_detected is True:
                    detected_in_row = detected_in_row + 1
                prev_detected = True
            else:
                prev_detected = False

        if detected_in_row >= self.NUM_OF_SCANS_TO_DETECT:
            self.is_barrier_1_detected = True

    def _detect_2_block(self):
        detected_in_row = 0
        prev_detected = True
        # Iterate over all elements of mem buffer
        for history in self.seq_2_memory_values:
            if self._check_if_obj_right(history):
                if prev_detected is True:
                    detected_in_row = detected_in_row + 1
                prev_detected = True
            else:
                prev_detected = False

        if detected_in_row >= self.NUM_OF_SCANS_TO_DETECT:
            self.is_barrier_2_detected = True

    # Saves values from right to mem buff

    def _filter_right_values(self, detected_objects: dict):
        # A dict of distances detected on right of the Lidar. (azimuth - dist)
        rightside_objects = dict()
        # Filter the input to only see objects on the right (Degrees 265 - 275)
        for key, value in detected_objects.items():
            if int(key) >= 794 and int(key) <= 824:
                rightside_objects[key] = value

        # Save sorted values to mem buffer
        self._add_to_memory_buffer(rightside_objects)

    def _check_if_obj_right(self, inp_values: dict) -> bool:
        # Check if more than 10 items are detected in the right side parking space
        if len(inp_values) > self.SCAN_LASER_WIDTH:
            output = True
        else:
            output = False

        return output

    def _check_for_empty_space(self, input_values: dict) -> bool:
        output = False
        if len(input_values) == 0:
            output = True

        return output

    def _filter_dict_by_key_range(self, input_dict, start_key, end_key):
        # A dict of distances detected on right of the Lidar. (azimuth - dist)
        output = dict()
        for key, value in input_dict.items():
            if int(key) >= start_key and int(key) <= end_key:
                output[key] = value

        return output
