from rclpy.node import Node
from std_msgs.msg import String
from sensor_msgs.msg import LaserScan

from .pedestrian_methods import is_pedestrian_detected
from ..utils import ObjectDetection, HorizontalLineSubscriber

import rclpy


class PedestrianCrossingNode(Node, HorizontalLineSubscriber):
    LINE_TYPE = 'zebra'

    def __init__(self):
        Node.__init__(self, 'pedestrian_crossing_node')
        HorizontalLineSubscriber.__init__(
            self,
            clock=self.get_clock(),
            create_client=self.create_client,
            create_timer=self.create_timer,
            logger=self.get_logger()
        )
        self.get_logger().info('PedestrianCrossingNode started')

        # Subscribe to the /horizontal_line_detect topic
        self.create_subscription(
            String,
            '/horizontal_line_detect',
            self._horizontal_line_detect_callback,
            10
        )

        # Subscribe to the /scan topic
        self.create_subscription(
            LaserScan,
            '/scan',
            self._scan_callback,
            10
        )

        self._object_detection = ObjectDetection()

    def _scan_callback(self, msg: LaserScan) -> None:
        # Return if the node is not 'working'
        if not self.is_working():
            return

        if (len(msg.ranges) != 1080):
            return

        self._object_detection.detect_objects(msg)
        # self.get_logger().info(self._object_detection.detection_location.value)

        can_drive = not is_pedestrian_detected(
            self._object_detection.get_most_frequent_position(self._object_detection._objects_detected)
        )
        self.send_kart_move_request(can_drive)


def main(args=None):
    rclpy.init(args=args)
    node = PedestrianCrossingNode()
    rclpy.spin(node)
    node.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
