from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.actions import IncludeLaunchDescription
from launch import LaunchDescription

from ament_index_python.packages import get_package_share_directory

import os


def generate_launch_description():
    # Set up the environment variable for the real robot
    os.environ['GAZEBO_SIMULATION'] = 'false'

    # Get the package directories
    maverick_control_dir = get_package_share_directory('control')
    maverick_core_dir = get_package_share_directory('core')
    maverick_perception_dir = get_package_share_directory('perception')

    # Define launch files
    control_launch_file = os.path.join(
        maverick_control_dir, 'launch', 'all_control.launch.py'
    )
    core_launch_file = os.path.join(
        maverick_core_dir, 'launch', 'all_core.launch.py'
    )
    perception_launch_file = os.path.join(
        maverick_perception_dir, 'launch', 'all_perception.launch.py'
    )

    # Create launch description
    ld = LaunchDescription()

    # Include gazebo launch file
    gazebo_launch = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(control_launch_file)
    )
    ld.add_action(gazebo_launch)

    # Include control launch file
    control_launch = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(core_launch_file)
    )
    ld.add_action(control_launch)

    # Include perception launch file
    perception_launch = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(perception_launch_file)
    )
    ld.add_action(perception_launch)

    return ld
