from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.actions import IncludeLaunchDescription
from launch import LaunchDescription

from ament_index_python.packages import get_package_share_directory

import os


def generate_launch_description():
    # Set up the environment variable for the simulation
    os.environ['GAZEBO_SIMULATION'] = 'true'

    # Get the package directories
    maverick_gazebo_dir = get_package_share_directory('gazebo_sim')
    maverick_core_dir = get_package_share_directory('core')

    # Define launch files
    gazebo_sim_launch_file = os.path.join(
        maverick_gazebo_dir, 'launch', 'rsp_sim.launch.py'
    )
    core_launch_file = os.path.join(
        maverick_core_dir, 'launch', 'all_core.launch.py'
    )

    # Create launch description
    ld = LaunchDescription()

    # Include gazebo launch file
    gazebo_launch = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(gazebo_sim_launch_file)
    )
    ld.add_action(gazebo_launch)

    # Include control launch file
    control_launch = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(core_launch_file)
    )
    ld.add_action(control_launch)

    return ld
