from launch import LaunchDescription
from launch.actions import IncludeLaunchDescription
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch_ros.actions import Node
from ament_index_python import get_package_share_directory

import xacro
import os


def generate_launch_description():
    os.environ['GAZEBO_SIMULATION'] = 'true'

    # Specify the path of the package and the path to the xarco file within the package
    pkg_name = 'gazebo_sim'
    xacro_file_sub_path = 'urdf/maverick.urdf.xacro'

    # Use xarco to process the file
    xacro_file = os.path.join(
        get_package_share_directory(pkg_name), xacro_file_sub_path)
    robot_description_raw = xacro.process_file(
        xacro_file).toxml()  # type: ignore

    # Configure the node
    node_robot_state_publisher = Node(
        package='robot_state_publisher',
        executable='robot_state_publisher',
        name='robot_state_publisher',
        output='screen',
        parameters=[{
            'robot_description': robot_description_raw,
            'use_sim_time': True
        }]
    )

    # Gazebo
    gazebo_params_file = os.path.join(
        get_package_share_directory(
            'gazebo_sim'
        ), 'config', 'gazebo_params.yaml'
    )

    gazebo_world = os.path.join(
        get_package_share_directory(
            'gazebo_sim'
        ), 'worlds', 'maverick_plane.world'
    )

    # Define the arguments to pass to the Gazebo launch file
    gazebo_arguments = {
        'world': gazebo_world,
        'extra_gazebo_args': f'--ros-args --params-file {gazebo_params_file}'
    }

    # Include the Gazebo launch file with the specified arguments
    gazebo_launch = IncludeLaunchDescription(
        PythonLaunchDescriptionSource([
            os.path.join(get_package_share_directory('gazebo_ros'), 'launch'),
            '/gazebo.launch.py'
        ]),
        launch_arguments=gazebo_arguments.items()
    )

    spawn_entity = Node(
        package='gazebo_ros',
        executable='spawn_entity.py',
        arguments=[
            '-topic', 'robot_description', '-entity',
            'maverick', '-x', '-36', '-y', '0.4', '-z', '0.25',
        ],
        output='screen'
    )

    ackermann_controller_spawner = Node(
        package='controller_manager',
        executable='spawner',
        arguments=[
            'ackermann_controller',
        ],
        output='screen'
    )

    joint_broad_spawner = Node(
        package='controller_manager',
        executable='spawner',
        arguments=['joint_broad'],
        output='screen'
    )

    # Start the cmd_vel_translation node
    cmd_vel_translation_node = Node(
        package='gazebo_sim',
        executable='cmd_vel_translation',
        name='cmd_vel_translation',
        output='screen'
    )

    # Run the node
    return LaunchDescription([
        # gazebo,
        gazebo_launch,
        node_robot_state_publisher,
        spawn_entity,
        ackermann_controller_spawner,
        joint_broad_spawner,
        cmd_vel_translation_node
    ])
