from typing import TypedDict, Optional
from functools import cache

import os
import json
import glob


class CamConfig(TypedDict):
    front: str
    left: str
    right: str


@cache
def get_camera_config(config_folder: str = "config") -> Optional[CamConfig]:
    """
    Select a valid camera configuration from the config_folder and return it.

    Returns None if no valid configuration could be found.
    """

    # Get the absolute path of the current file
    file_path = os.path.dirname(os.path.realpath(__file__))
    error_message = "No valid camera configuration found in the config folder."

    files = glob.glob(
        os.path.join(
            file_path, os.path.normpath(config_folder), '*.json'
        )
    )

    for filename in files:
        with open(filename, 'r') as fp:
            config: CamConfig = json.load(fp)
            if all((os.path.exists(camera) and not os.path.isdir(camera) for camera in config.values())):
                return config

    if len(files) == 0:
        error_message = "No file found in the config folder."

    raise FileNotFoundError(error_message)
